@@ -71,8 +71,7 @@ Last update: 2017-05-16
 #include <time.h>
 
 #include "proj_internal.h"
-#include "proj_math.h"
-#include "proj_internal.h"
+#include <math.h>
 
 PROJ_HEAD(unitconvert, "Unit conversion");
 
@@ -471,13 +470,13 @@ PJ *CONVERSION(unitconvert,0) {
         const char* normalized_name = nullptr;
         f = get_unit_conversion_factor(name, &xy_in_is_linear, &normalized_name);
         if (f != 0.0) {
-            proj_log_debug(P, "xy_in unit: %s", normalized_name);
+            proj_log_trace(P, "xy_in unit: %s", normalized_name);
         } else {
-            if ( (f = pj_param (P->ctx, P->params, "dxy_in").f) == 0.0)
+            f = pj_param (P->ctx, P->params, "dxy_in").f;
+            if (f == 0.0 || 1.0 / f == 0.0)
                 return pj_default_destructor(P, PJD_ERR_UNKNOWN_UNIT_ID);
         }
-        if (f != 0.0)
-            Q->xy_factor *= f;
+        Q->xy_factor = f;
         if (normalized_name != nullptr && strcmp(normalized_name, "Radian") == 0)
             P->left = PJ_IO_UNITS_RADIANS;
     }
@@ -486,13 +485,13 @@ PJ *CONVERSION(unitconvert,0) {
         const char* normalized_name = nullptr;
         f = get_unit_conversion_factor(name, &xy_out_is_linear, &normalized_name);
         if (f != 0.0) {
-            proj_log_debug(P, "xy_out unit: %s", normalized_name);
+            proj_log_trace(P, "xy_out unit: %s", normalized_name);
         } else {
-            if ( (f = pj_param (P->ctx, P->params, "dxy_out").f) == 0.0)
+            f = pj_param (P->ctx, P->params, "dxy_out").f;
+            if (f == 0.0 || 1.0 / f == 0.0)
                 return pj_default_destructor(P, PJD_ERR_UNKNOWN_UNIT_ID);
         }
-        if (f != 0.0)
-            Q->xy_factor /= f;
+        Q->xy_factor /= f;
         if (normalized_name != nullptr && strcmp(normalized_name, "Radian") == 0)
             P->right= PJ_IO_UNITS_RADIANS;
     }
@@ -507,26 +506,26 @@ PJ *CONVERSION(unitconvert,0) {
         const char* normalized_name = nullptr;
         f = get_unit_conversion_factor(name, &z_in_is_linear, &normalized_name);
         if (f != 0.0) {
-            proj_log_debug(P, "z_in unit: %s", normalized_name);
+            proj_log_trace(P, "z_in unit: %s", normalized_name);
         } else {
-            if ( (f = pj_param (P->ctx, P->params, "dz_in").f) == 0.0)
+            f = pj_param (P->ctx, P->params, "dz_in").f;
+            if (f == 0.0 || 1.0 / f == 0.0)
                 return pj_default_destructor(P, PJD_ERR_UNKNOWN_UNIT_ID);
         }
-        if (f != 0.0)
-            Q->z_factor *= f;
+        Q->z_factor = f;
     }
 
     if ((name = pj_param (P->ctx, P->params, "sz_out").s) != nullptr) {
         const char* normalized_name = nullptr;
         f = get_unit_conversion_factor(name, &z_out_is_linear, &normalized_name);
         if (f != 0.0) {
-            proj_log_debug(P, "z_out unit: %s", normalized_name);
+            proj_log_trace(P, "z_out unit: %s", normalized_name);
         } else {
-            if ( (f = pj_param (P->ctx, P->params, "dz_out").f) == 0.0)
+            f = pj_param (P->ctx, P->params, "dz_out").f;
+            if (f == 0.0 || 1.0 / f == 0.0)
                 return pj_default_destructor(P, PJD_ERR_UNKNOWN_UNIT_ID);
         }
-        if (f != 0.0)
-            Q->z_factor /= f;
+        Q->z_factor /= f;
     }
 
     if( z_in_is_linear >= 0 && z_out_is_linear >= 0 &&
@@ -541,7 +540,7 @@ PJ *CONVERSION(unitconvert,0) {
         if (!s) return pj_default_destructor(P, PJD_ERR_UNKNOWN_UNIT_ID); /* unknown unit conversion id */
 
         Q->t_in_id = i;
-        proj_log_debug(P, "t_in unit: %s", time_units[i].name);
+        proj_log_trace(P, "t_in unit: %s", time_units[i].name);
     }
 
     s = nullptr;
@@ -551,7 +550,7 @@ PJ *CONVERSION(unitconvert,0) {
         if (!s) return pj_default_destructor(P, PJD_ERR_UNKNOWN_UNIT_ID); /* unknown unit conversion id */
 
         Q->t_out_id = i;
-        proj_log_debug(P, "t_out unit: %s", time_units[i].name);
+        proj_log_trace(P, "t_out unit: %s", time_units[i].name);
     }
 
     return P;
